classdef soundFourierMaxHold < matlab.apps.AppBase

    %
    %
    %          SoundFourierMaxHold App        
    %   Copyright 2020 Thomas P Weldon
    %
    %  UI allowing user to change frequency and amplitude
    %
    % Properties that correspond to app components
    properties (Access = public)
        SoundFourierMaxHoldApp            matlab.ui.Figure
        GridLayout               matlab.ui.container.GridLayout
        LeftPanel                matlab.ui.container.Panel
        FrequencyLabel           matlab.ui.control.Label
        freqEditField            matlab.ui.control.NumericEditField
        AmplitudedBLabel         matlab.ui.control.Label
        amplitudeEditField       matlab.ui.control.NumericEditField
        FreqkHzSliderLabel       matlab.ui.control.Label
        FreqkHzSlider            matlab.ui.control.Slider
        AmplSliderLabel          matlab.ui.control.Label
        AmplSlider               matlab.ui.control.Slider
        RightPanel               matlab.ui.container.Panel
        MyAxesUIAxes  matlab.ui.control.UIAxes
        MaxButton               matlab.ui.control.Button
        StartButton              matlab.ui.control.Button
        RateButton              matlab.ui.control.Button

    sampleRate=44100
    samplesPerFrame=1024*8
    sineSource 
    deviceWriter
    deviceReader
    frequency=0
    amplitude=10^(-20/20)
    startbut=0
    maxbut=0
    maxyfft=0
    maxfreq=44100/2
    end

    % Properties that correspond to apps with auto-reflow
    properties (Access = private)
        onePanelWidth = 576;
    end

    % Callbacks that handle component events
    methods (Access = private)

        % Changes arrangement of the app based on UIFigure width
        function updateAppLayout(app, event)
            currentFigureWidth = app.SoundFourierMaxHoldApp.Position(3);
            if(currentFigureWidth <= app.onePanelWidth)
                % Change to a 2x1 grid
                app.GridLayout.RowHeight = {316, 316};
                app.GridLayout.ColumnWidth = {'1x'};
                app.RightPanel.Layout.Row = 2;
                app.RightPanel.Layout.Column = 1;
            else
                % Change to a 1x2 grid
                app.GridLayout.RowHeight = {'1x'};
                app.GridLayout.ColumnWidth = {257, '1x'};
                app.RightPanel.Layout.Row = 1;
                app.RightPanel.Layout.Column = 2;
            end
        end

        function createSineSource(app)
            app.sineSource  = audioOscillator( ...
                  'Frequency', app.freqEditField.Value, ...
                   'Amplitude', 10^(app.amplitudeEditField.Value/20), ...
                    'SignalType', 'sine', ...
                     'SampleRate', app.sampleRate, ...
                    'SamplesPerFrame', 2*app.samplesPerFrame);        
        end
        
        function createDeviceWriter(app)
            app.deviceWriter = audioDeviceWriter;
            getAudioDevices(app.deviceWriter)
            %app.deviceWriter.Device='S111 BT';   % bluetooth speaker purchased for this course
            app.deviceWriter.Device='Default';
            %deviceWriter.Device='Built-in Output';
            app.deviceWriter.SampleRate=app.sampleRate;
            app.deviceWriter.ChannelMappingSource='Property';
            app.deviceWriter.ChannelMapping=1;

        end
 
           
%microphone input

       function  createDeviceReader(app)
            app.deviceReader = audioDeviceReader;
            getAudioDevices(app.deviceReader)
%             app.deviceReader.Device='Built-in Microphone'; %
             app.deviceReader.Device='Default'; %
           app.deviceReader.SampleRate=app.sampleRate;
            app.deviceReader.SamplesPerFrame=app.samplesPerFrame;
            app.deviceReader.ChannelMappingSource='Property';
            app.deviceReader.ChannelMapping=1;

       end
 


        % Value changed function: freqEditField
        function FreqBoxChanged(app, event)
           
        end
        
  
   
  
        % Value changed function: amplitudeEditField
        function AmplitudeBoxChanged(app, event)
      
           
        end
 
        
         
        % Button pushed function: startButton
        function StartButtonCallback(app, event)
            if app.startbut==0;
                app.startbut=1;
            else
                app.startbut=0;
            end
            startbutton=app.startbut
                        
             %clear input
             yy=app.deviceReader();yy=app.deviceReader();
            yy=app.deviceReader();yy=app.deviceReader();
            yy=app.deviceReader();yy=app.deviceReader();

            
            n=0
            while 1
                if app.startbut==0 ;
                    app.maxbut=0
                    break
                end
                n=n+1
                yy=app.deviceReader();

                maxin=max(yy)
                yfft=fft(yy);
                yfft=yfft/length(yfft);
                yfft=20*log10(abs(yfft));
                nx=[1:length(yfft)];
                frq=(nx-1)/app.samplesPerFrame*app.sampleRate;

           if app.maxbut==0;
                app.maxyfft=yfft;
            else
                app.maxyfft=max(yfft,app.maxyfft);
            end

                [maxfft maxindex]=max(app.maxyfft)
                maxfft=maxfft
                maxfreq=frq(maxindex)
%  disp("app.freqEditField.Value",num2str(round(maxfreq)))
%  disp(round(maxfreq))
% disp("app.amplitudeEditField.Value",num2str(round(10*maxfft)/10))
% disp(round(10*maxfft)/10)
% disp("maxfreq")
% disp(maxfreq)
% disp("maxfft")
% disp(maxfft)
%         pause(1)                   
                app.freqEditField.Value =round(maxfreq); 
                app.amplitudeEditField.Value = round(10*maxfft)/10;
%  disp("app.amplitudeEditField.Value",num2str(app.amplitudeEditField.Value))
%  disp("app.freqEditField.Value",num2str(app.freqEditField.Value))
%  pause(1)               
   
                %app.MyAxesUIAxes.YLim = [max(fy)-80 max(fy)];
                plot(app.MyAxesUIAxes, frq, app.maxyfft,'LineWidth',3) ;
                pause(0.2)

            end  
        end
    
        % Button pushed function: MaxButton
        function MaxButtonCallback(app, event)
           if app.maxbut==0
                app.maxbut=1
            else
                app.maxbut=0;
           end
            
        end
        
        % Button pushed function: MaxButton
        function RateButtonCallback(app, event)
           if app.maxfreq==app.sampleRate/2
                app.maxfreq=1000
                app.RateButton.Text = [ 'Max Freq. = ' num2str(app.maxfreq) ];
%                 release(app.deviceWriter);
%                 release(app.deviceReader);
                pause(0.2)
                %createDeviceWriter(app)
%                 app.createDeviceWriter();
%                 app.createDeviceReader();
%                app.deviceReader.SampleRate=app.sampleRate;
                app.MyAxesUIAxes.XLim = [0 app.maxfreq];
                app.MyAxesUIAxes.XTick = [0  app.maxfreq];
                app.MyAxesUIAxes.XTickLabel = {'0'; num2str(app.maxfreq) };

            else
                app.maxfreq=app.sampleRate/2
                app.RateButton.Text = [ 'Max Freq. = ' num2str(app.maxfreq) ];
%                 release(app.deviceWriter);
%                 release(app.deviceReader);
                pause(0.2)
                %createDeviceWriter(app)
%                 app.createDeviceWriter();
%                 app.createDeviceReader();
%                app.deviceReader.SampleRate=app.sampleRate;
                app.MyAxesUIAxes.XLim = [0 app.maxfreq];
                app.MyAxesUIAxes.XTick = [0  app.maxfreq];
                app.MyAxesUIAxes.XTickLabel = {'0'; num2str(app.maxfreq) };
           end
            
        end

   
    
    end

    % Component initialization
    methods (Access = private)

        % Create UIFigure and components
        function createComponents(app)

            % Create SoundFourierMaxHoldApp and hide until all components are created
            app.SoundFourierMaxHoldApp = uifigure('Visible', 'off');
            app.SoundFourierMaxHoldApp.AutoResizeChildren = 'off';
            app.SoundFourierMaxHoldApp.Position = [100 100 653 316];
            app.SoundFourierMaxHoldApp.Name = 'Sound Fourier Max Hold         Copyright 2020 Thomas P Weldon';
            app.SoundFourierMaxHoldApp.SizeChangedFcn = createCallbackFcn(app, @updateAppLayout, true);

            % Create GridLayout
            app.GridLayout = uigridlayout(app.SoundFourierMaxHoldApp);
            app.GridLayout.ColumnWidth = {257, '1x'};
            app.GridLayout.RowHeight = {'1x'};
            app.GridLayout.ColumnSpacing = 0;
            app.GridLayout.RowSpacing = 0;
            app.GridLayout.Padding = [0 0 0 0];
            app.GridLayout.Scrollable = 'on';

            % Create LeftPanel
            app.LeftPanel = uipanel(app.GridLayout);
            app.LeftPanel.Layout.Row = 1;
            app.LeftPanel.Layout.Column = 1;
            app.LeftPanel.Scrollable = 'on';


            % Create MaxButton
            app.MaxButton = uibutton(app.LeftPanel, 'push');
            app.MaxButton.ButtonPushedFcn = createCallbackFcn(app, @MaxButtonCallback, true);
            %app.MaxButton.Position = [13 285 100 22];
            app.MaxButton.Position = [100 250 100 22];
            app.MaxButton.Text = 'Max Hold';

            % Create startButton
            app.StartButton = uibutton(app.LeftPanel, 'push');
            app.StartButton.ButtonPushedFcn = createCallbackFcn(app, @StartButtonCallback, true);
            %app.StartButton.Position = [142 264 100 22];
            app.StartButton.Position = [100 200 100 22];
            app.StartButton.Text = 'Start/Stop';

            % Create rateButton
            app.RateButton = uibutton(app.LeftPanel, 'push');
            app.RateButton.ButtonPushedFcn = createCallbackFcn(app, @RateButtonCallback, true);
            %app.StartButton.Position = [142 264 100 22];
            app.RateButton.Position = [50 20 150 22];
            app.RateButton.Text = [ 'Max Freq. = ' num2str(app.sampleRate/2) ];

 
            % Create FrequencyLabel
            app.FrequencyLabel = uilabel(app.LeftPanel);
            app.FrequencyLabel.HorizontalAlignment = 'right';
            app.FrequencyLabel.Position = [25 150 88 22];
            app.FrequencyLabel.Text = 'Max Freq.';

            % Create freqEditField
            app.freqEditField = uieditfield(app.LeftPanel, 'numeric');
            app.freqEditField.Limits = [0 22100];
            app.freqEditField.ValueDisplayFormat = '%8.f';
 %           app.freqEditField.ValueChangedFcn = createCallbackFcn(app, @FreqBoxChanged, true);
            app.freqEditField.Position = [125 150 100 22];
            app.freqEditField.Value = 150;

            % Create AmplitudedBLabel
            app.AmplitudedBLabel = uilabel(app.LeftPanel);
            app.AmplitudedBLabel.HorizontalAlignment = 'right';
            app.AmplitudedBLabel.Position = [25 100 88 22];
            app.AmplitudedBLabel.Text = 'Max. dB';

            % Create amplitudeEditField
            app.amplitudeEditField = uieditfield(app.LeftPanel, 'numeric');
            app.amplitudeEditField.Limits = [-1000 1000];
 %           app.amplitudeEditField.ValueChangedFcn = createCallbackFcn(app, @AmplitudeBoxChanged, true);
            app.amplitudeEditField.Position = [125 100 100 22];
            app.amplitudeEditField.Value = -20;

% 
            % Create RightPanel
            app.RightPanel = uipanel(app.GridLayout);
            app.RightPanel.Layout.Row = 1;
            app.RightPanel.Layout.Column = 2;
            app.RightPanel.Scrollable = 'on';

            % Create MyAxesUIAxes
            app.MyAxesUIAxes = uiaxes(app.RightPanel);
            title(app.MyAxesUIAxes, 'Sound Level dB')
            xlabel(app.MyAxesUIAxes, 'Frequency')
            ylabel(app.MyAxesUIAxes, 'Amplitude dB')
            app.MyAxesUIAxes.XLim = [0 app.sampleRate/2];
            app.MyAxesUIAxes.YLim = [20*log10(abs(app.samplesPerFrame))-200 20*log10(abs(app.samplesPerFrame))];
            app.MyAxesUIAxes.XTick = [0  app.sampleRate/2];
            app.MyAxesUIAxes.XTickLabel = {'0'; num2str(app.sampleRate/2) };
            app.MyAxesUIAxes.XGrid = 'on';
            app.MyAxesUIAxes.YGrid = 'on';
            app.MyAxesUIAxes.Position = [30 36 326 250];

            % Show the figure after all components are created
            app.SoundFourierMaxHoldApp.Visible = 'on';
        end
    end
    


    % App creation and deletion
    methods (Access = public)

        % Construct app
        function app = soundFourierMaxHold

            % Create UIFigure and components
            createComponents(app)

            % Register the app with App Designer
            registerApp(app, app.SoundFourierMaxHoldApp)
            
            xx=[app.freqEditField.Value/1000 app.freqEditField.Value/1000]
            yy=[-100 app.AmplSlider.Value]
            plot(app.MyAxesUIAxes, xx, yy,'LineWidth',4) ;
            
                app.maxfreq=app.sampleRate/2;
                app.RateButton.Text = [ 'Max Freq. = ' num2str(app.maxfreq) ];
                app.MyAxesUIAxes.XLim = [0 app.maxfreq];
                app.MyAxesUIAxes.XTick = [0  app.maxfreq];
                app.MyAxesUIAxes.XTickLabel = {'0'; num2str(app.maxfreq) };
     
        createSineSource(app)
        createDeviceReader(app)
        createDeviceWriter(app)
 
  
  
            if nargout == 0
                clear app
            end
        end

        % Code that executes before app deletion
        function delete(app)
            
            release(app.deviceReader)
            release(app.deviceWriter)


            % Delete UIFigure when app is deleted
            delete(app.SoundFourierMaxHoldApp)
        end
    end
end